import 'package:flutter_test/flutter_test.dart';
import 'package:kyc/Components/document_type.dart';
import 'package:kyc/Exceptions/sdk_exceptions.dart';
import 'package:kyc/Helper/api_helper.dart';
import 'package:kyc/Models/kyc_document.dart';
import 'package:kyc/Services/call_api_service.dart';
import 'package:kyc/Services/document_service.dart';
import 'package:kyc/Utils/channel_utils.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';

import 'document_service_test.mocks.dart';

@GenerateMocks([ChannelUtils, ApiHelper])
void main() {
  setUpAll(() {
    TestWidgetsFlutterBinding.ensureInitialized();
  });

  group('DocumentService for sdktoken test', (){
    test("Get SdkTokenMissingException when sdktoken is empty", () async {
      ChannelUtils mockedChannel = MockChannelUtils();
      CallApiService callApiService = new CallApiService.withChannel(mockedChannel);
      when(mockedChannel.getEnvironment())
          .thenAnswer((_) => Future.value("PROD"));
      await callApiService.init();
      when(mockedChannel.getSdkToken())
      .thenAnswer((_) => Future.value(""));

      DocumentService documentService = new DocumentService.withService(callApiService);
      expect(() => documentService.sendDocument(null,"USA"), throwsA(isA<SdkTokenMissingException>()));
    });

    test("No exception when sdktoken is not empty", () async {
      ApiHelper mockedApi = MockApiHelper();
      ChannelUtils mockedChannel = MockChannelUtils();
      when(mockedChannel.getEnvironment())
          .thenAnswer((_) => Future.value("PROD"));
      CallApiService callApiService = new CallApiService.withChannelApiHelper(mockedChannel, mockedApi);
      KycDocument document = new KycDocument();
      document.type = DocumentType.FaceScan;
      String token = "real_token";

      when(mockedChannel.getSdkToken())
          .thenAnswer((_) => Future.value(token));
      when(mockedApi.uploadFace(document))
          .thenAnswer((_) => Future.value("Result"));
      when(mockedApi.setSdkToken(token)).thenAnswer((realInvocation) { });

      DocumentService documentService = new DocumentService.withService(callApiService);
      expect(await documentService.sendDocument(document, "USA"), "Result");
    });

    test("Problem on getEnvironment in ChanneUtils should throw EnvironmentMissingException", () async {
      new DocumentService().sendDocument(null,"USA");
    });
  });
}